@@ -2,18 +2,16 @@
 #define H_MISC
 
 /**
- * \file lib/misc.h
+ * \file misc.h
  *
  */
 
 #include <string.h>
 #include <rpm/rpmtypes.h>
 #include <rpm/header.h>		/* for headerGetFlags typedef, duh.. */
-#include "lib/rpmfs.h"
+#include "rpmfs.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+typedef const struct headerFmt_s * headerFmt;
 
 /* known arch? */
 RPM_GNUC_INTERNAL
@@ -23,19 +21,21 @@ RPM_GNUC_INTERNAL
 char * rpmVerifyString(uint32_t verifyResult, const char *pad);
 
 RPM_GNUC_INTERNAL
-char * rpmFFlagsString(uint32_t fflags, const char *pad);
+char * rpmFFlagsString(uint32_t fflags);
 
-typedef char * (*headerTagFormatFunction) (rpmtd td);
 typedef int (*headerTagTagFunction) (Header h, rpmtd td, headerGetFlags hgflags);
 
 RPM_GNUC_INTERNAL
 headerTagTagFunction rpmHeaderTagFunc(rpmTagVal tag);
 
 RPM_GNUC_INTERNAL
-headerTagFormatFunction rpmHeaderFormatFuncByName(const char *fmt);
+headerFmt rpmHeaderFormatByName(const char *fmt);
 
 RPM_GNUC_INTERNAL
-headerTagFormatFunction rpmHeaderFormatFuncByValue(rpmtdFormats fmt);
+headerFmt rpmHeaderFormatByValue(rpmtdFormats fmt);
+
+RPM_GNUC_INTERNAL
+char * rpmHeaderFormatCall(headerFmt fmt, rpmtd td);
 
 RPM_GNUC_INTERNAL
 int headerFindSpec(Header h);
@@ -58,8 +58,4 @@ RPM_GNUC_INTERNAL
 void rpmRelocationBuild(Header h, rpmRelocation *rawrelocs,
 		int *rnrelocs, rpmRelocation **rrelocs, uint8_t **rbadrelocs);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif	/* H_MISC */
