@@ -9,8 +9,8 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmds.h>
 
-#include "lib/rpmte_internal.h"	/* XXX tsortInfo_s */
-#include "lib/rpmts_internal.h"
+#include "rpmte_internal.h"	/* XXX tsortInfo_s */
+#include "rpmts_internal.h"
 
 #include "debug.h"
 
@@ -67,12 +67,13 @@ static void rpmTSIFree(tsortInfo tsi)
 
 static inline int addSingleRelation(rpmte p,
 				    rpmte q,
-				    rpmsenseFlags dsflags,
-				    int reversed)
+				    rpmds dep)
 {
     struct tsortInfo_s *tsi_p, *tsi_q;
     relation rel;
     rpmElementType teType = rpmteType(p);
+    rpmsenseFlags dsflags = rpmdsFlags(dep);
+    int reversed = rpmdsIsReverse(dep);
     rpmsenseFlags flags;
 
     /* Avoid deps outside this transaction and self dependencies */
@@ -93,6 +94,22 @@ static inline int addSingleRelation(rpmte p,
 	    RPMSENSE_SCRIPT_PRE : RPMSENSE_SCRIPT_PREUN;
     }
 
+    /*
+     * Avoid dependency loop tangles from weak dependencies: demote scriptlet
+     * dependencies to regular ones to avoid loop-breaker inflation, and
+     * and ignore non-scriptlet ones entirely (like "meta" would do).
+     */
+    if (rpmdsIsWeak(dep)) {
+	/* ...but demoting ordering hints would be tragicomic */
+	if (rpmdsTagN(dep) != RPMTAG_ORDERNAME) {
+	    if (flags) {
+		flags = 0;
+	    } else {
+		return 0;
+	    }
+	}
+    }
+
     if (reversed) {
 	rpmte r = p;
 	p = q;
@@ -139,7 +156,7 @@ static inline int addSingleRelation(rpmte p,
     /* bump p predecessor count */
     tsi_p->tsi_count++;
 
-    rel = xcalloc(1, sizeof(*rel));
+    rel = (relation)xcalloc(1, sizeof(*rel));
     rel->rel_suc = tsi_p;
     rel->rel_flags = flags;
 
@@ -150,7 +167,7 @@ static inline int addSingleRelation(rpmte p,
     /* bump q successor count */
     tsi_q->tsi_qcnt++;
 
-    rel = xcalloc(1, sizeof(*rel));
+    rel = (relation)xcalloc(1, sizeof(*rel));
     rel->rel_suc = tsi_q;
     rel->rel_flags = flags;
 
@@ -165,53 +182,49 @@ static inline int addSingleRelation(rpmte p,
  * @param ts		transaction set
  * @param al		packages list
  * @param p		predecessor (i.e. package that "Requires: q")
- * @param requires	relation
+ * @param dep		dependency relation
  * @return		0 always
  */
 static inline int addRelation(rpmts ts,
 			      rpmal al,
 			      rpmte p,
-			      rpmds requires,
-			      int reversed)
+			      rpmds dep)
 {
     rpmte q;
-    rpmsenseFlags dsflags;
-
-    dsflags = rpmdsFlags(requires);
 
     /* Avoid dependendencies which are not relevant for ordering */
-    if (isUnorderedReq(rpmdsFlags(requires)))
+    if (isUnorderedReq(rpmdsFlags(dep)))
 	return 0;
 
-    if (rpmdsIsRich(requires)) {
+    if (rpmdsIsRich(dep)) {
 	rpmds ds1, ds2;
 	rpmrichOp op;
-	if (rpmdsParseRichDep(requires, &ds1, &ds2, &op, NULL) == RPMRC_OK) {
+	if (rpmdsParseRichDep(dep, &ds1, &ds2, &op, NULL) == RPMRC_OK) {
 	    if (op != RPMRICHOP_ELSE)
-		addRelation(ts, al, p, ds1, reversed);
+		addRelation(ts, al, p, ds1);
 	    if (op == RPMRICHOP_IF || op == RPMRICHOP_UNLESS) {
 	      rpmds ds21, ds22;
 	      rpmrichOp op2;
-	      if (rpmdsParseRichDep(requires, &ds21, &ds22, &op2, NULL) == RPMRC_OK && op2 == RPMRICHOP_ELSE) {
-		  addRelation(ts, al, p, ds22, reversed);
+	      if (rpmdsParseRichDep(dep, &ds21, &ds22, &op2, NULL) == RPMRC_OK && op2 == RPMRICHOP_ELSE) {
+		  addRelation(ts, al, p, ds22);
 	      }
 	      ds21 = rpmdsFree(ds21);
 	      ds22 = rpmdsFree(ds22);
 	    }
 	    if (op == RPMRICHOP_AND || op == RPMRICHOP_OR)
-		addRelation(ts, al, p, ds2, reversed);
+		addRelation(ts, al, p, ds2);
 	    ds1 = rpmdsFree(ds1);
 	    ds2 = rpmdsFree(ds2);
 	}
 	return 0;
     }
-    q = rpmalSatisfiesDepend(al, p, requires);
+    q = rpmalSatisfiesDepend(al, p, dep);
 
     /* Avoid deps outside this transaction and self dependencies */
     if (q == NULL || q == p)
 	return 0;
 
-    addSingleRelation(p, q, dsflags, reversed);
+    addSingleRelation(p, q, dep);
 
     return 0;
 }
@@ -219,8 +232,8 @@ static inline int addRelation(rpmts ts,
 /**
  * Add element to list sorting by tsi_qcnt.
  * @param p		new element
- * @retval qp		address of first element
- * @retval rp		address of last element
+ * @param[out] qp		address of first element
+ * @param[out] rp		address of last element
  * @param prefcolor
  */
 static void addQ(tsortInfo p, tsortInfo * qp, tsortInfo * rp,
@@ -337,7 +350,8 @@ static void tarjan(sccData sd, tsortInfo tsi)
 	    } while (tsi_q != tsi);
 	    sd->SCCs[sd->sccCnt].size = sd->stackcnt - stackIdx;
 	    /* copy members */
-	    sd->SCCs[sd->sccCnt].members = xcalloc(sd->SCCs[sd->sccCnt].size,
+	    sd->SCCs[sd->sccCnt].members =
+				(tsortInfo *)xcalloc(sd->SCCs[sd->sccCnt].size,
 					   sizeof(tsortInfo));
 	    memcpy(sd->SCCs[sd->sccCnt].members, sd->stack + stackIdx,
 		   sd->SCCs[sd->sccCnt].size * sizeof(tsortInfo));
@@ -351,8 +365,8 @@ static void tarjan(sccData sd, tsortInfo tsi)
 static scc detectSCCs(tsortInfo orderInfo, int nelem, int debugloops)
 {
     /* Set up data structures needed for the tarjan algorithm */
-    scc SCCs = xcalloc(nelem+3, sizeof(*SCCs));
-    tsortInfo *stack = xcalloc(nelem, sizeof(*stack));
+    scc SCCs = (scc)xcalloc(nelem+3, sizeof(*SCCs));
+    tsortInfo *stack = (tsortInfo *)xcalloc(nelem, sizeof(*stack));
     struct sccData_s sd = { 0, stack, 0, SCCs, 2 };
 
     for (int i = 0; i < nelem; i++) {
@@ -455,7 +469,7 @@ static void dijkstra(const struct scc_s *SCC, int sccNr)
     relation rel;
 
     /* can use a simple queue as edge weights are always 1 */
-    tsortInfo * queue = xmalloc((SCC->size+1) * sizeof(*queue));
+    tsortInfo * queue = (tsortInfo *)xmalloc((SCC->size+1) * sizeof(*queue));
 
     /*
      * Find packages that are prerequired and use them as
@@ -570,7 +584,7 @@ int rpmtsOrder(rpmts ts)
     rpmal erasedPackages;
     scc SCCs;
     int nelem = rpmtsNElements(ts);
-    tsortInfo sortInfo = xcalloc(nelem, sizeof(struct tsortInfo_s));
+    tsortInfo sortInfo = (tsortInfo)xcalloc(nelem, sizeof(struct tsortInfo_s));
 
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_ORDER), 0);
 
@@ -588,51 +602,38 @@ int rpmtsOrder(rpmts ts)
     while ((p = rpmtsiNext(pi, 0)) != NULL) {
 	rpmal al = (rpmteType(p) == TR_REMOVED) ? 
 		   erasedPackages : tsmem->addedPackages;
-	rpmds requires = rpmdsInit(rpmteDS(p, RPMTAG_REQUIRENAME));
-	rpmds recommends = rpmdsInit(rpmteDS(p, RPMTAG_RECOMMENDNAME));
-	rpmds suggests = rpmdsInit(rpmteDS(p, RPMTAG_SUGGESTNAME));
-	rpmds supplements = rpmdsInit(rpmteDS(p, RPMTAG_SUPPLEMENTNAME));
-	rpmds enhances = rpmdsInit(rpmteDS(p, RPMTAG_ENHANCENAME));
-	rpmds order = rpmdsInit(rpmteDS(p, RPMTAG_ORDERNAME));
+	rpmTagVal ordertags[] = {
+		RPMTAG_REQUIRENAME,
+		RPMTAG_RECOMMENDNAME,
+		RPMTAG_SUGGESTNAME,
+		RPMTAG_SUPPLEMENTNAME,
+		RPMTAG_ENHANCENAME,
+		RPMTAG_ORDERNAME,
+		0,
+	};
 
-	while (rpmdsNext(requires) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" requires "q"). */
-	    (void) addRelation(ts, al, p, requires, 0);
-	}
-
-	while (rpmdsNext(recommends) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" recommends "q"). */
-	    (void) addRelation(ts, al, p, recommends, 0);
-	}
-
-	while (rpmdsNext(suggests) >= 0) {
-	    /* Record next "q <- p" relation (i.e. "p" suggests "q"). */
-	    (void) addRelation(ts, al, p, suggests, 0);
-	}
-
-	while (rpmdsNext(order) >= 0) {
-	    /* Record next "q <- p" ordering request */
-	    (void) addRelation(ts, al, p, order, 0);
-	}
-
-	while (rpmdsNext(supplements) >= 0) {
-	    /* Record next "p -> q" relation (i.e. "q" supplemented by "p"). */
-	    (void) addRelation(ts, al, p, supplements, 1);
-	}
-
-	while (rpmdsNext(enhances) >= 0) {
-	    /* Record next "p <- q" relation (i.e. "q" is enhanced by  "p"). */
-	    (void) addRelation(ts, al, p, enhances, 1);
+	for (int i = 0; ordertags[i]; i++) {
+	    rpmds dep = rpmdsInit(rpmteDS(p, ordertags[i]));
+	    while (rpmdsNext(dep) >= 0)
+		addRelation(ts, al, p, dep);
 	}
     }
 
     rpmtsiFree(pi);
 
-    newOrder = xcalloc(tsmem->orderCount, sizeof(*newOrder));
+    newOrder = (rpmte*)xcalloc(tsmem->orderCount, sizeof(*newOrder));
     SCCs = detectSCCs(sortInfo, nelem, (rpmtsFlags(ts) & RPMTRANS_FLAG_DEPLOOPS));
 
     rpmlog(RPMLOG_DEBUG, "========== tsorting packages (order, #predecessors, #succesors, depth)\n");
 
+    /* Restored items first (doesn't matter but is simple) */
+    for (int e = 0; e < nelem; e++) {
+	tsortInfo p = &sortInfo[e];
+	if (rpmteType(p->te) == TR_RESTORED) {
+	    newOrder[newOrderCount++] = p->te;
+	}
+    }
+
     for (int i = 0; i < 2; i++) {
 	/* Do two separate runs: installs first - then erases */
 	int oType = !i ? TR_ADDED : TR_REMOVED;
