@@ -8,15 +8,12 @@
 #include <rpm/rpmstring.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmstrpool.h>
+#include <rpm/rpmbase64.h>
 
-#include "lib/rpmds_internal.h"
+#include "rpmds_internal.h"
 
 #include "debug.h"
 
-int _rpmds_debug = 0;
-
-int _rpmds_nopromote = 1;
-
 /**
  * A package dependency set.
  */
@@ -32,7 +29,6 @@ struct rpmds_s {
     int32_t Count;		/*!< No. of elements */
     unsigned int instance;	/*!< From rpmdb instance? */
     int i;			/*!< Element index. */
-    int nopromote;		/*!< Don't promote Epoch: in rpmdsCompare()? */
     int nrefs;			/*!< Reference count. */
     int *ti;			/*!< Trigger index. */
 };
@@ -42,7 +38,7 @@ struct depinfo_s {
     rpmTagVal evrTag;
     rpmTagVal flagTag;
     rpmTagVal ixTag;
-    char *name;
+    const char *name;
     char abrev;
 };
 
@@ -254,14 +250,13 @@ static rpmds rpmdsCreate(rpmstrPool pool,
 		  rpmTagVal tagN, const char * Type, int Count,
 		  unsigned int instance)
 {
-    rpmds ds = xcalloc(1, sizeof(*ds));
+    rpmds ds = (rpmds)xcalloc(1, sizeof(*ds));
 
     ds->pool = (pool != NULL) ? rpmstrPoolLink(pool) : rpmstrPoolCreate();
     ds->tagN = tagN;
     ds->Type = Type;
     ds->Count = Count;
     ds->instance = instance;
-    ds->nopromote = _rpmds_nopromote;
     ds->i = -1;
 
     return rpmdsLink(ds);
@@ -277,7 +272,7 @@ rpmds rpmdsNewPool(rpmstrPool pool, Header h, rpmTagVal tagN, int flags)
 	goto exit;
 
     if (headerGet(h, tagN, &names, HEADERGET_MINMEM)) {
-	struct rpmtd_s evr, flags, tindices;
+	struct rpmtd_s evr, dflags, tindices;
 	rpm_count_t count = rpmtdCount(&names);
 
 	headerGet(h, tagEVR, &evr, HEADERGET_MINMEM);
@@ -286,9 +281,9 @@ rpmds rpmdsNewPool(rpmstrPool pool, Header h, rpmTagVal tagN, int flags)
 	    return NULL;
 	}
 
-	headerGet(h, tagF, &flags, HEADERGET_ALLOC);
-	if (flags.count && flags.count != count) {
-	    rpmtdFreeData(&flags);
+	headerGet(h, tagF, &dflags, HEADERGET_ALLOC);
+	if (dflags.count && dflags.count != count) {
+	    rpmtdFreeData(&dflags);
 	    return NULL;
 	}
 
@@ -304,9 +299,9 @@ rpmds rpmdsNewPool(rpmstrPool pool, Header h, rpmTagVal tagN, int flags)
 
 	ds->N = names.count ? rpmtdToPool(&names, ds->pool) : NULL;
 	ds->EVR = evr.count ? rpmtdToPool(&evr, ds->pool): NULL;
-	ds->Flags = flags.data;
+	ds->Flags = (uint32_t *)dflags.data;
 	if (tagTi != RPMTAG_NOT_FOUND) {
-	    ds->ti = tindices.data;
+	    ds->ti = (int *)tindices.data;
 	}
 
 	/* ensure rpmlib() requires always have RPMSENSE_RPMLIB flag set */
@@ -360,7 +355,7 @@ char * rpmdsNewDNEVR(const char * dspfx, const rpmds ds)
 	nb += strlen(EVR);
     }
 
-    t = tbuf = xmalloc(nb + 1);
+    t = tbuf = (char *)xmalloc(nb + 1);
     if (dspfx) {
 	t = stpcpy(t, dspfx);
 	*t++ = ' ';
@@ -397,14 +392,14 @@ static rpmds singleDSPool(rpmstrPool pool, rpmTagVal tagN,
 
     ds = rpmdsCreate(pool, tagN, Type, 1, instance);
 
-    ds->N = xmalloc(1 * sizeof(*ds->N));
+    ds->N = (rpmsid *)xmalloc(1 * sizeof(*ds->N));
     ds->N[0] = N;
-    ds->EVR = xmalloc(1 * sizeof(*ds->EVR));
+    ds->EVR = (rpmsid *)xmalloc(1 * sizeof(*ds->EVR));
     ds->EVR[0] = EVR;
-    ds->Flags = xmalloc(sizeof(*ds->Flags));
+    ds->Flags = (uint32_t *)xmalloc(sizeof(*ds->Flags));
     ds->Flags[0] = Flags;
     if (tagTi != RPMTAG_NOT_FOUND) {
-	ds->ti = xmalloc(sizeof(*ds->ti));
+	ds->ti = (int *)xmalloc(sizeof(*ds->ti));
 	ds->ti[0] = triggerIndex;
     }
     ds->i = 0;
@@ -498,10 +493,10 @@ rpmds rpmdsFilterTi(rpmds ds, int ti)
 
     fds = rpmdsCreate(ds->pool, ds->tagN, ds->Type, tiCount, ds->instance);
 
-    fds->N = xmalloc(tiCount * sizeof(*fds->N));
-    fds->EVR = xmalloc(tiCount * sizeof(*fds->EVR));
-    fds->Flags = xmalloc(tiCount * sizeof(*fds->Flags));
-    fds->ti = xmalloc(tiCount * sizeof(*fds->ti));
+    fds->N = (rpmsid *)xmalloc(tiCount * sizeof(*fds->N));
+    fds->EVR = (rpmsid *)xmalloc(tiCount * sizeof(*fds->EVR));
+    fds->Flags = (uint32_t *)xmalloc(tiCount * sizeof(*fds->Flags));
+    fds->ti = (int *)xmalloc(tiCount * sizeof(*fds->ti));
     fds->i = -1;
 
     i2 = 0;
@@ -555,10 +550,10 @@ int rpmdsSetIx(rpmds ds, int ix)
 {
     int i = -1;
 
-    if (ds != NULL) {
-	i = ds->i;
+    if (ds != NULL && ix >= 0 && ix < ds->Count) {
 	ds->i = ix;
 	ds->DNEVR = _free(ds->DNEVR);
+	i = ds->i;
     }
     return i;
 }
@@ -657,26 +652,6 @@ unsigned int rpmdsInstance(rpmds ds)
     return (ds != NULL) ? ds->instance : 0;
 }
 
-int rpmdsNoPromote(const rpmds ds)
-{
-    int nopromote = 0;
-
-    if (ds != NULL)
-	nopromote = ds->nopromote;
-    return nopromote;
-}
-
-int rpmdsSetNoPromote(rpmds ds, int nopromote)
-{
-    int onopromote = 0;
-
-    if (ds != NULL) {
-	onopromote = ds->nopromote;
-	ds->nopromote = nopromote;
-    }
-    return onopromote;
-}
-
 rpm_color_t rpmdsColor(const rpmds ds)
 {
     return (ds != NULL) ? rpmdsColorIndex(ds, ds->i) : 0;
@@ -688,7 +663,7 @@ rpm_color_t rpmdsSetColor(const rpmds ds, rpm_color_t color)
 
     if (ds != NULL && ds->i >= 0 && ds->i < ds->Count) {
 	if (ds->Color == NULL) {
-	    ds->Color = xcalloc(ds->Count, sizeof(*ds->Color));
+	    ds->Color = (uint32_t *)xcalloc(ds->Count, sizeof(*ds->Color));
 	}
 	ocolor = ds->Color[ds->i];
 	ds->Color[ds->i] = color;
@@ -717,17 +692,8 @@ int rpmdsNext(rpmds ds)
 {
     int i = -1;
 
-    if (ds != NULL && ++ds->i >= 0) {
-	if (ds->i < ds->Count) {
-	    i = ds->i;
-	    ds->DNEVR = _free(ds->DNEVR);
-	} else
-	    ds->i = -1;
-
-if (_rpmds_debug  < 0 && i != -1)
-fprintf(stderr, "*** ds %p\t%s[%d]: %s\n", ds, (ds->Type ? ds->Type : "?Type?"), i, (ds->DNEVR ? ds->DNEVR : "?DNEVR?"));
-
-    }
+    if (ds != NULL)
+	i = rpmdsSetIx(ds, ds->i + 1);
 
     return i;
 }
@@ -748,25 +714,24 @@ static rpmds rpmdsDup(const rpmds ods)
     size_t nb;
     
     ds->i = ods->i;
-    ds->nopromote = ods->nopromote;
 
     nb = ds->Count * sizeof(*ds->N);
-    ds->N = memcpy(xmalloc(nb), ods->N, nb);
+    ds->N = (rpmsid *)memcpy(xmalloc(nb), ods->N, nb);
     
     /* XXX rpm prior to 3.0.2 did not always supply EVR and Flags. */
     if (ods->EVR) {
 	nb = ds->Count * sizeof(*ds->EVR);
-	ds->EVR = memcpy(xmalloc(nb), ods->EVR, nb);
+	ds->EVR = (rpmsid *)memcpy(xmalloc(nb), ods->EVR, nb);
     }
 
     if (ods->Flags) {
 	nb = ds->Count * sizeof(*ds->Flags);
-	ds->Flags = memcpy(xmalloc(nb), ods->Flags, nb);
+	ds->Flags = (uint32_t *)memcpy(xmalloc(nb), ods->Flags, nb);
     }
 
     if (ods->ti) {
 	nb = ds->Count * sizeof(*ds->ti);
-	ds->ti = memcpy(xmalloc(nb), ods->ti, nb);
+	ds->ti = (int *)memcpy(xmalloc(nb), ods->ti, nb);
     }
 
     return ds;
@@ -848,12 +813,12 @@ int rpmdsMerge(rpmds * dsp, rpmds ods)
 
     /* Ensure EVR and Flags exist */
     if (ds->EVR == NULL)
-	ds->EVR = xcalloc(ds->Count, sizeof(*ds->EVR));
+	ds->EVR = (rpmsid *)xcalloc(ds->Count, sizeof(*ds->EVR));
     if (ds->Flags == NULL)
-	ds->Flags = xcalloc(ds->Count, sizeof(*ds->Flags));
+	ds->Flags = (uint32_t *)xcalloc(ds->Count, sizeof(*ds->Flags));
     if (ds->ti == NULL && ods->ti) {
 	int i;
-	ds->ti = xcalloc(ds->Count, sizeof(*ds->ti));
+	ds->ti = (int *)xcalloc(ds->Count, sizeof(*ds->ti));
 	for (i = 0; i < ds->Count; i++)
 	    ds->ti[i] = -1;
     }
@@ -958,123 +923,17 @@ int rpmdsSearch(rpmds ds, rpmds ods)
     /* Check each member of [l,u) subset for ranges overlap. */
     i = -1;
     if (l < u) {
-	int save = rpmdsSetIx(ds, l-1);
-	while ((l = rpmdsNext(ds)) >= 0 && (l < u)) {
-	    if ((i = rpmdsCompare(ods, ds)) != 0)
+	int oix = rpmdsIx(ods);
+	while (l < u) {
+	    if (rpmdsCompareIndex(ods, oix, ds, l) != 0) {
+		i = l;
 		break;
-	}
-	/* Return element index that overlaps, or -1. */
-	if (i)
-	    i = rpmdsIx(ds);
-	else {
-	    (void) rpmdsSetIx(ds, save);
-	    i = -1;
+	    }
+	    l++;
 	}
     }
     return i;
 }
-/**
- * Split EVR into epoch, version, and release components.
- * @param evr		[epoch:]version[-release] string
- * @retval *ep		pointer to epoch
- * @retval *vp		pointer to version
- * @retval *rp		pointer to release
- */
-static
-void parseEVR(char * evr,
-		const char ** ep,
-		const char ** vp,
-		const char ** rp)
-{
-    const char *epoch;
-    const char *version;		/* assume only version is present */
-    const char *release;
-    char *s, *se;
-
-    s = evr;
-    while (*s && risdigit(*s)) s++;	/* s points to epoch terminator */
-    se = strrchr(s, '-');		/* se points to version terminator */
-
-    if (*s == ':') {
-	epoch = evr;
-	*s++ = '\0';
-	version = s;
-	if (*epoch == '\0') epoch = "0";
-    } else {
-	epoch = NULL;	/* XXX disable epoch compare if missing */
-	version = evr;
-    }
-    if (se) {
-	*se++ = '\0';
-	release = se;
-    } else {
-	release = NULL;
-    }
-
-    if (ep) *ep = epoch;
-    if (vp) *vp = version;
-    if (rp) *rp = release;
-}
-
-static inline int rpmdsCompareEVR(const char *AEVR, uint32_t AFlags,
-				  const char *BEVR, uint32_t BFlags,
-				  int nopromote)
-{
-    const char *aE, *aV, *aR, *bE, *bV, *bR;
-    char *aEVR = xstrdup(AEVR);
-    char *bEVR = xstrdup(BEVR);
-    int sense = 0;
-    int result = 0;
-
-    parseEVR(aEVR, &aE, &aV, &aR);
-    parseEVR(bEVR, &bE, &bV, &bR);
-
-    /* Compare {A,B} [epoch:]version[-release] */
-    if (aE && *aE && bE && *bE)
-	sense = rpmvercmp(aE, bE);
-    else if (aE && *aE && atol(aE) > 0) {
-	if (!nopromote) {
-	    sense = 0;
-	} else
-	    sense = 1;
-    } else if (bE && *bE && atol(bE) > 0)
-	sense = -1;
-
-    if (sense == 0) {
-	sense = rpmvercmp(aV, bV);
-	if (sense == 0) {
-	    if (aR && *aR && bR && *bR) {
-		sense = rpmvercmp(aR, bR);
-	    } else {
-		/* always matches if the side with no release has SENSE_EQUAL */
-		if ((aR && *aR && (BFlags & RPMSENSE_EQUAL)) ||
-		    (bR && *bR && (AFlags & RPMSENSE_EQUAL))) {
-		    aEVR = _free(aEVR);
-		    bEVR = _free(bEVR);
-		    result = 1;
-		    goto exit;
-		}
-	    }
-	}
-    }
-
-    /* Detect overlap of {A,B} range. */
-    if (sense < 0 && ((AFlags & RPMSENSE_GREATER) || (BFlags & RPMSENSE_LESS))) {
-	result = 1;
-    } else if (sense > 0 && ((AFlags & RPMSENSE_LESS) || (BFlags & RPMSENSE_GREATER))) {
-	result = 1;
-    } else if (sense == 0 &&
-	(((AFlags & RPMSENSE_EQUAL) && (BFlags & RPMSENSE_EQUAL)) ||
-	 ((AFlags & RPMSENSE_LESS) && (BFlags & RPMSENSE_LESS)) ||
-	 ((AFlags & RPMSENSE_GREATER) && (BFlags & RPMSENSE_GREATER)))) {
-	result = 1;
-    }
-
-exit:
-    free(aEVR);
-    free(bEVR);
-    return result;
-}
 
 int rpmdsCompareIndex(rpmds A, int aix, rpmds B, int bix)
 {
@@ -1110,7 +969,13 @@ int rpmdsCompareIndex(rpmds A, int aix, rpmds B, int bix)
 	result = 1;
     } else {
 	/* Both AEVR and BEVR exist, compare [epoch:]version[-release]. */
-	result = rpmdsCompareEVR(AEVR, AFlags, BEVR, BFlags, B->nopromote);
+	rpmver av = rpmverParse(AEVR);
+	rpmver bv = rpmverParse(BEVR);
+
+	result = rpmverOverlap(av, AFlags, bv, BFlags);
+
+	rpmverFree(av);
+	rpmverFree(bv);
     }
 
 exit:
@@ -1123,7 +988,7 @@ int rpmdsCompare(const rpmds A, const rpmds B)
 }
 
 int rpmdsMatches(rpmstrPool pool, Header h, int prix,
-		 rpmds req, int selfevr, int nopromote)
+		 rpmds req, int selfevr)
 {
     rpmds provides;
     rpmTagVal tag = RPMTAG_PROVIDENAME;
@@ -1135,8 +1000,6 @@ int rpmdsMatches(rpmstrPool pool, Header h, int prix,
     else
 	provides = rpmdsNewPool(pool, h, tag, 0);
 
-    rpmdsSetNoPromote(provides, nopromote);
-
     /*
      * For a self-provide and indexed provide, we only need one comparison.
      * Otherwise loop through the provides until match or end.
@@ -1161,17 +1024,17 @@ int rpmdsMatches(rpmstrPool pool, Header h, int prix,
 
 int rpmdsMatchesDep (const Header h, int ix, const rpmds req, int nopromote)
 {
-    return rpmdsMatches(NULL, h, ix, req, 0, nopromote);
+    return rpmdsMatches(NULL, h, ix, req, 0);
 }
 
 int rpmdsAnyMatchesDep (const Header h, const rpmds req, int nopromote)
 {
-    return rpmdsMatches(NULL, h, -1, req, 0, nopromote);
+    return rpmdsMatches(NULL, h, -1, req, 0);
 }
 
 int rpmdsNVRMatchesDep(const Header h, const rpmds req, int nopromote)
 {
-    return rpmdsMatches(NULL, h, -1, req, 1, nopromote);
+    return rpmdsMatches(NULL, h, -1, req, 1);
 }
 
 /**
@@ -1190,12 +1053,12 @@ static const struct rpmlibProvides_s rpmlibProvides[] = {
     { "rpmlib(CompressedFileNames)",	"3.0.4-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
     N_("file name(s) stored as (dirName,baseName,dirIndex) tuple, not as path.")},
-#if HAVE_BZLIB_H
+#ifdef HAVE_BZLIB_H
     { "rpmlib(PayloadIsBzip2)",		"3.0.5-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
     N_("package payload can be compressed using bzip2.") },
 #endif
-#if HAVE_LZMA_H
+#ifdef HAVE_LZMA_H
     { "rpmlib(PayloadIsXz)",		"5.2-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
     N_("package payload can be compressed using xz.") },
@@ -1221,11 +1084,9 @@ static const struct rpmlibProvides_s rpmlibProvides[] = {
     { "rpmlib(ConcurrentAccess)",    "4.1-1",
 	(                RPMSENSE_EQUAL),
     N_("package scriptlets may access the rpm database while installing.") },
-#ifdef WITH_LUA
     { "rpmlib(BuiltinLuaScripts)",    "4.2.2-1",
 	(                RPMSENSE_EQUAL),
     N_("internal support for lua scripts.") },
-#endif
     { "rpmlib(FileDigests)", 		"4.6.0-1",
 	(		 RPMSENSE_EQUAL),
     N_("file digest algorithm is per package configurable") },
@@ -1263,7 +1124,7 @@ static const struct rpmlibProvides_s rpmlibProvides[] = {
 
 int rpmdsRpmlibPool(rpmstrPool pool, rpmds * dsp, const void * tblp)
 {
-    const struct rpmlibProvides_s * rltblp = tblp;
+    const struct rpmlibProvides_s * rltblp = (const struct rpmlibProvides_s *)tblp;
     const struct rpmlibProvides_s * rlp;
     int rc = 0;
 
@@ -1319,6 +1180,32 @@ int rpmdsIsReverse(rpmds ds)
     return reverse;
 }
 
+int rpmdsIsSysuser(rpmds ds, char **sysuser)
+{
+    if (rpmdsTagN(ds) != RPMTAG_PROVIDENAME)
+	return 0;
+    if (!(rpmdsFlags(ds) & RPMSENSE_EQUAL))
+	return 0;
+
+    const char *name = rpmdsN(ds);
+    if (!(rstreqn(name, "user(", 5) || rstreqn(name, "group(", 6) ||
+	    rstreqn(name, "groupmember(", 12))) {
+	return 0;
+    }
+
+    char *line = NULL;
+    size_t llen = 0;
+
+    if (rpmBase64Decode(rpmdsEVR(ds), (void **)&line, &llen))
+	return 0;
+
+    if (sysuser)
+	*sysuser = rstrndup(line, llen);
+    free(line);
+
+    return 1;
+}
+
 rpmsenseFlags rpmSanitizeDSFlags(rpmTagVal tagN, rpmsenseFlags Flags)
 {
     rpmsenseFlags extra = RPMSENSE_ANY;
@@ -1336,6 +1223,7 @@ rpmsenseFlags rpmSanitizeDSFlags(rpmTagVal tagN, rpmsenseFlags Flags)
     case RPMTAG_SUPPLEMENTNAME:
     case RPMTAG_ENHANCENAME:
     case RPMTAG_REQUIRENAME:
+    case RPMTAG_ORDERNAME:
 	extra = Flags & (_ALL_REQUIRES_MASK);
 	break;
     case RPMTAG_CONFLICTNAME:
@@ -1385,7 +1273,7 @@ static struct RichOpComp {
     { "else",	 RPMRICHOP_ELSE},
     { "with",	 RPMRICHOP_WITH},
     { "without", RPMRICHOP_WITHOUT},
-    { NULL, 0 },
+    { NULL, RPMRICHOP_NONE },
 };
 
 int rpmdsIsRich(rpmds dep)
@@ -1507,7 +1395,7 @@ static rpmRC rpmrichParseCheck(rpmrichOp op, int check, char **emsg)
 static rpmRC rpmrichParseInternal(const char **dstrp, char **emsg, rpmrichParseFunction cb, void *cbdata, int *checkp)
 {
     const char *p = *dstrp, *pe;
-    rpmrichOp op = RPMRICHOP_SINGLE, firstop = RPMRICHOP_SINGLE, chainop = 0;
+    rpmrichOp op = RPMRICHOP_SINGLE, firstop = RPMRICHOP_SINGLE, chainop = RPMRICHOP_NONE;
     int check = checkp ? *checkp : 0;
 
     if (cb && cb(cbdata, RPMRICH_PARSE_ENTER, p, 0, 0, 0, 0, op, emsg) != RPMRC_OK)
@@ -1554,7 +1442,7 @@ static rpmRC rpmrichParseInternal(const char **dstrp, char **emsg, rpmrichParseF
 	    firstop = op;
 
 	if (op == RPMRICHOP_ELSE && (chainop == RPMRICHOP_IF || chainop == RPMRICHOP_UNLESS))
-	    chainop = 0;
+	    chainop = RPMRICHOP_NONE;
         if (chainop && op != chainop) {
             if (emsg)
                 rasprintf(emsg, _("Cannot chain different ops"));
@@ -1635,7 +1523,7 @@ struct rpmdsParseRichDepData {
 static rpmRC rpmdsParseRichDepCB(void *cbdata, rpmrichParseType type,
 		const char *n, int nl, const char *e, int el, rpmsenseFlags sense,
 		rpmrichOp op, char **emsg) {
-    struct rpmdsParseRichDepData *data = cbdata;
+    struct rpmdsParseRichDepData *data = (struct rpmdsParseRichDepData *)cbdata;
     rpmds ds = 0;
 
     if (type == RPMRICH_PARSE_ENTER)
@@ -1643,7 +1531,7 @@ static rpmRC rpmdsParseRichDepCB(void *cbdata, rpmrichParseType type,
     else if (type == RPMRICH_PARSE_LEAVE) {
 	if (--data->depth == 0 && data->dochain && data->rightstart) {
 	    /* chain op hack, construct a sub-ds from the right side of the chain */
-	    char *right = xmalloc(n + nl - data->rightstart + 2);
+	    char *right = (char *)xmalloc(n + nl - data->rightstart + 2);
 	    right[0] = '(';
 	    strncpy(right + 1, data->rightstart, n + nl - data->rightstart);
 	    right[n + nl - data->rightstart + 1] = 0;
