@@ -3,7 +3,7 @@
 
 #include <rpm/rpmtypes.h>
 #include <rpm/rpmts.h> /* for rpmVSFlags */
-#include "lib/header_internal.h"
+#include "header_internal.h"
 
 /* siginfo range bits */
 enum {
@@ -19,6 +19,7 @@ struct rpmsinfo_s {
     /* parsed data */
     int hashalgo;
     int sigalgo;
+    int alt;
     int id;
     int wrapped;
     int strength;
@@ -44,10 +45,6 @@ struct rpmsinfo_s {
  */
 typedef int (*rpmsinfoCb)(struct rpmsinfo_s *sinfo, void *cbdata);
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 const char *rpmsinfoDescr(struct rpmsinfo_s *sinfo);
 
 char *rpmsinfoMsg(struct rpmsinfo_s *sinfo);
@@ -74,8 +71,4 @@ int rpmvsVerify(struct rpmvs_s *sis, int type,
 rpmRC rpmpkgRead(struct rpmvs_s *vs, FD_t fd,
 		hdrblob *sigblobp, hdrblob *blobp, char **emsg);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif /* _RPMVS_H */
