@@ -13,9 +13,9 @@
 #include <rpm/rpmlog.h>
 #include <rpm/rpmmacro.h>
 
-#include "lib/rpmlead.h"
-#include "lib/header_internal.h"
-#include "lib/signature.h"
+#include "rpmlead.h"
+#include "header_internal.h"
+#include "signature.h"
 
 #include "debug.h"
 
@@ -65,7 +65,7 @@ rpmRC rpmReadSignature(FD_t fd, Header * sighp, char ** msg)
     if (sighp)
 	*sighp = NULL;
 
-    if (hdrblobRead(fd, 1, 1, RPMTAG_HEADERSIGNATURES, &blob, &buf) != RPMRC_OK)
+    if (hdrblobRead(fd, 1, 0, RPMTAG_HEADERSIGNATURES, &blob, &buf) != RPMRC_OK)
 	goto exit;
     
     /* OK, blob looks sane, load the header. */
@@ -115,9 +115,11 @@ rpmRC rpmGenerateSignature(char *SHA256, char *SHA1, uint8_t *MD5,
     Header sig = headerNew();
     struct rpmtd_s td;
     rpmRC rc = RPMRC_OK;
-    char *reservedSpace;
+    uint8_t *reservedSpace;
     int spaceSize = 32; /* always reserve a bit of space */
     int gpgSize = rpmExpandNumeric("%{__gpg_reserved_space}");
+    rpm_off_t size32 = size;
+    rpm_off_t payloadSize32 = payloadSize;
 
     /* Prepare signature */
     if (SHA256) {
@@ -149,21 +151,34 @@ rpmRC rpmGenerateSignature(char *SHA256, char *SHA1, uint8_t *MD5,
 
     rpmtdReset(&td);
     td.count = 1;
-    if (payloadSize < UINT32_MAX) {
-	rpm_off_t p = payloadSize;
-	rpm_off_t s = size;
-	td.type = RPM_INT32_TYPE;
+    td.type = RPM_INT32_TYPE;
 
-	td.tag = RPMSIGTAG_PAYLOADSIZE;
-	td.data = &p;
-	headerPut(sig, &td, HEADERPUT_DEFAULT);
+    td.tag = RPMSIGTAG_PAYLOADSIZE;
+    td.data = &payloadSize32;
+    headerPut(sig, &td, HEADERPUT_DEFAULT);
 
-	td.tag = RPMSIGTAG_SIZE;
-	td.data = &s;
-	headerPut(sig, &td, HEADERPUT_DEFAULT);
-    } else {
+    td.tag = RPMSIGTAG_SIZE;
+    td.data = &size32;
+    headerPut(sig, &td, HEADERPUT_DEFAULT);
+
+    if (size >= UINT32_MAX || payloadSize >= UINT32_MAX) {
+	/*
+	 * Put the 64bit size variants into the header, but
+	 * modify spaceSize so that the resulting header has
+	 * the same size. Note that this only works if all tags
+	 * with a lower number than RPMSIGTAG_RESERVEDSPACE are
+	 * already added and no tag with a higher number is
+	 * added yet.
+	 */
 	rpm_loff_t p = payloadSize;
 	rpm_loff_t s = size;
+	int newsigSize, oldsigSize;
+
+	oldsigSize = headerSizeof(sig, HEADER_MAGIC_YES);
+
+	headerDel(sig, RPMSIGTAG_PAYLOADSIZE);
+	headerDel(sig, RPMSIGTAG_SIZE);
+
 	td.type = RPM_INT64_TYPE;
 
 	td.tag = RPMSIGTAG_LONGARCHIVESIZE;
@@ -174,15 +189,15 @@ rpmRC rpmGenerateSignature(char *SHA256, char *SHA1, uint8_t *MD5,
 	td.data = &s;
 	headerPut(sig, &td, HEADERPUT_DEFAULT);
 
-	/* adjust for the size difference between 64- and 32bit tags */
-	spaceSize -= 8;
+	newsigSize = headerSizeof(sig, HEADER_MAGIC_YES);
+	spaceSize -= newsigSize - oldsigSize;
     }
 
     if (gpgSize > 0)
 	spaceSize += gpgSize;
 
     if (spaceSize > 0) {
-	reservedSpace = xcalloc(spaceSize, sizeof(char));
+	reservedSpace = (uint8_t *)xcalloc(spaceSize, sizeof(char));
 	rpmtdReset(&td);
 	td.tag = RPMSIGTAG_RESERVEDSPACE;
 	td.count = spaceSize;
